package main

import (
	"context"
	"crypto/sha256"
	"errors"
	"fmt"
	"log"

	"git.c3pb.de/gbe/invinoveritas/auth"
	"github.com/jmoiron/sqlx"
)

func hashPassword(ctx context.Context, db *sqlx.DB, user, pass string) (string, error) {
	// Look up password salt from DB and hash password with it
	query := `SELECT val FROM state WHERE key = 'pwsalt'`

	var salt []byte
	err := db.GetContext(ctx, &salt, query)
	if err != nil {
		return "", err
	}

	h := sha256.New()

	_, err = h.Write(salt)
	if err != nil {
		return "", err
	}

	fmt.Fprint(h, ":", user, ":", pass)

	return fmt.Sprintf("%02x", h.Sum(nil)), nil
}

type authProvider struct {
	db *sqlx.DB
}

func (a authProvider) Valid(ctx context.Context, userName, pass string) (*auth.User, error) {
	query := `SELECT count(*) FROM users;`

	var count int
	err := a.db.GetContext(ctx, &count, query)
	if err != nil {
		return nil, err
	}

	user := &auth.User{
		Name: userName,
	}

	// If no entry in user DB, just allow everything, so that initial user creation works.
	if count == 0 {
		log.Println("user database empty, allowing access by", user, "regardless of password")
		return user, nil
	}

	hashedPw, err := hashPassword(ctx, a.db, userName, pass)

	log.Printf("hashed pw for %s: %s", userName, hashedPw)

	query = `SELECT rowid, name FROM users WHERE name = ? AND password = ?`

	err = a.db.GetContext(ctx, user, query, userName, hashedPw)
	if err != nil {
		return nil, err
	}

	return user, nil
}

func (a authProvider) updatePassword(ctx context.Context, userName, passOld, passNew string) error {
	hashedOld, err := hashPassword(ctx, a.db, userName, passOld)
	if err != nil {
		return err
	}

	hashedNew, err := hashPassword(ctx, a.db, userName, passNew)
	if err != nil {
		return err
	}

	const query = `UPDATE users SET password = ? WHERE name = ? AND password = ?`

	res, err := a.db.ExecContext(ctx, query, hashedNew, userName, hashedOld)
	if err != nil {
		return err
	}

	n, err := res.RowsAffected()
	if err != nil {
		return err
	}

	if n == 0 {
		return errors.New("authentication failed")
	}

	log.Println("pw update result:", n, err)

	return nil
}
